@@ -23,6 +23,7 @@
 #include <unistd.h>
 #include <dlfcn.h>
 #include <link.h>
+#include <pthread.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 
@@ -43,6 +44,8 @@ extern OSSL_LIB_CTX *openssl_libctx;
 extern OSSL_PROVIDER *openssl_provider;
 #endif
 
+extern pthread_rwlock_t fips_list_lock;
+
 extern int ica_stats_enabled;
 
 int openssl_in_fips_mode(void)
@@ -995,8 +998,9 @@ aes_gcm_kat(void) {
 		memset(tag, 0, AES_BLKSIZE);
 		memset(out, 0, tv->len);
 		memset(icb, 0, sizeof(icb));
-		memset(icb, 0, sizeof(ucb));
+		memset(ucb, 0, sizeof(ucb));
 		memset(subkey, 0, sizeof(subkey));
+		ica_allow_external_gcm_iv_in_fips_mode(1);
 		if ((tv->rv == 0) && (ica_aes_gcm_initialize_internal(tv->iv, tv->ivlen,
 		    tv->key, tv->keylen, icb, ucb, subkey, ICA_ENCRYPT)
 		    || ica_aes_gcm_intermediate(tv->plaintext,
@@ -1010,6 +1014,7 @@ aes_gcm_kat(void) {
 		    ICA_ENCRYPT) || memcmp(tv->ciphertext, out, tv->len)
 		    || memcmp(tv->tag, tag, tv->taglen)))
 			goto _err_;
+		ica_allow_external_gcm_iv_in_fips_mode(0);
 
 		free(tag);
 		free(out);
@@ -1017,6 +1022,7 @@ aes_gcm_kat(void) {
 	return 0;
 
 _err_:
+	ica_allow_external_gcm_iv_in_fips_mode(0);
 	free(tag);
 	free(out);
 #ifndef NO_CPACF
@@ -1240,9 +1246,6 @@ ecdsa_kat(void)
 		/* adapter handle not needed here, just CPACF */
 		rc = ica_ecdsa_sign_ex_internal(0, eckey, tv->hash, tv->hashlen,
 								sigbuf, tv->siglen, tv->k);
-		if (rc)
-			goto _err_;
-		rc = ica_ecdsa_verify(0, eckey, tv->hash, tv->hashlen, sigbuf, tv->siglen);
 		if (rc)
 			goto _err_;
 		if (memcmp(sigbuf, tv->sig, tv->siglen) != 0) {
@@ -1361,4 +1364,180 @@ _err_:
 	syslog(LOG_ERR, "Libica RSA test failed.");
 	return 1;
 }
+
+/*
+ * List of non-fips-approved algorithms
+ */
+int FIPS_BLACKLIST[] = {DES_ECB, DES_CBC, DES_CBC_CS, DES_OFB,
+	DES_CFB, DES_CTR, DES_CTRLST, DES_CBC_MAC, DES_CMAC, P_RNG, DES3_ECB,
+	DES3_CBC, DES3_CBC_CS, DES3_OFB, DES3_CFB, DES3_CTR, DES3_CTRLST,
+	DES3_CBC_MAC, DES3_CMAC, ED25519_KEYGEN, ED25519_SIGN, ED25519_VERIFY,
+	ED448_KEYGEN, ED448_SIGN, ED448_VERIFY, X25519_KEYGEN, X25519_DERIVE,
+	X448_KEYGEN, X448_DERIVE, RSA_ME, RSA_CRT, SHA512_DRNG, -1, -1 };
+const size_t FIPS_BLACKLIST_LEN
+	= sizeof(FIPS_BLACKLIST) / sizeof(FIPS_BLACKLIST[0]);
+
+/*
+ * FIPS service indicator: List of tolerated but non-approved algorithms.
+ */
+int FIPS_OVERRIDE_LIST[] = { RSA_ME, RSA_CRT, SHA512_DRNG, -1, -1 };
+const size_t FIPS_OVERRIDE_LIST_LEN
+	= sizeof(FIPS_OVERRIDE_LIST) / sizeof(FIPS_OVERRIDE_LIST[0]);
+
+/*
+ * Returns 1 if the algorithm identified by @id is FIPS approved.
+ * Returns 0 otherwise.
+ */
+int fips_approved(int id)
+{
+	size_t i;
+	int rc;
+
+	if (pthread_rwlock_rdlock(&fips_list_lock) != 0) {
+		syslog(LOG_ERR, "Error obtaining read-lock in fips_approved");
+		return 0;
+	}
+
+	for (i = 0; i < FIPS_BLACKLIST_LEN; i++) {
+		if (id == FIPS_BLACKLIST[i]) {
+			rc = 0;
+			goto done;
+		}
+	}
+
+	rc = 1;
+
+done:
+	if (pthread_rwlock_unlock(&fips_list_lock) != 0) {
+		syslog(LOG_ERR, "Error unlock read-lock in fips_approved");
+		return 0;
+	}
+
+	return rc;
+}
+
+/*
+ * Returns 1 if the algorithm identified by @id is FIPS tolerated, i.e. it is
+ * available via the API in fips mode, but considered non-approved.
+ * Returns 0 otherwise.
+ */
+int fips_override(int id)
+{
+	size_t i;
+	int rc;
+
+	if (pthread_rwlock_rdlock(&fips_list_lock) != 0) {
+		syslog(LOG_ERR, "Error obtaining read-lock in fips_override");
+		return 0;
+	}
+
+	for (i = 0; i < FIPS_OVERRIDE_LIST_LEN; i++) {
+		if (id == FIPS_OVERRIDE_LIST[i]) {
+			rc = 1;
+			goto done;
+		}
+	}
+
+	rc = 0;
+
+done:
+	if (pthread_rwlock_unlock(&fips_list_lock) != 0) {
+		syslog(LOG_ERR, "Error unlock read-lock in fips_override");
+		return 0;
+	}
+
+	return rc;
+}
+
+/*
+ * Following routines add an algo id to a fips list by replacing a
+ * placeholder indicated by -1 by the given id.
+ */
+void add_to_fips_black_list(int id)
+{
+	size_t i;
+
+	if (pthread_rwlock_wrlock(&fips_list_lock) != 0) {
+		syslog(LOG_ERR, "Error obtaining write-lock in add_to_fips_black_list");
+		return;
+	}
+
+	for (i = 0; i < FIPS_BLACKLIST_LEN; i++) {
+		if (FIPS_BLACKLIST[i] == -1) {
+			FIPS_BLACKLIST[i] = id;
+			goto done;
+		}
+	}
+
+done:
+	if (pthread_rwlock_unlock(&fips_list_lock) != 0)
+		syslog(LOG_ERR, "Error unlock write-lock in add_to_fips_black_list");
+}
+
+void add_to_fips_override_list(int id)
+{
+	size_t i;
+
+	if (pthread_rwlock_wrlock(&fips_list_lock) != 0) {
+		syslog(LOG_ERR, "Error obtaining write-lock in add_to_fips_override_list");
+		return;
+	}
+
+	for (i = 0; i < FIPS_OVERRIDE_LIST_LEN; i++) {
+		if (FIPS_OVERRIDE_LIST[i] == -1) {
+			FIPS_OVERRIDE_LIST[i] = id;
+			goto done;
+		}
+	}
+
+done:
+	if (pthread_rwlock_unlock(&fips_list_lock) != 0)
+		syslog(LOG_ERR, "Error unlock write-lock in add_to_fips_override_list");
+}
+
+/*
+ * Following routines remove an algo id from a fips list by replacing the
+ * algo id by the placeholder indicated by -1.
+ */
+void remove_from_fips_black_list(int id)
+{
+	size_t i;
+
+	if (pthread_rwlock_wrlock(&fips_list_lock) != 0) {
+		syslog(LOG_ERR, "Error obtaining write-lock in remove_from_fips_black_list");
+		return;
+	}
+
+	for (i = 0; i < FIPS_BLACKLIST_LEN; i++) {
+		if (FIPS_BLACKLIST[i] == id) {
+			FIPS_BLACKLIST[i] = -1;
+			goto done;
+		}
+	}
+
+done:
+	if (pthread_rwlock_unlock(&fips_list_lock) != 0)
+		syslog(LOG_ERR, "Error unlock write-lock in remove_from_fips_black_list");
+}
+
+void remove_from_fips_override_list(int id)
+{
+	size_t i;
+
+	if (pthread_rwlock_wrlock(&fips_list_lock) != 0) {
+		syslog(LOG_ERR, "Error obtaining write-lock in remove_from_fips_override_list");
+		return;
+	}
+
+	for (i = 0; i < FIPS_OVERRIDE_LIST_LEN; i++) {
+		if (FIPS_OVERRIDE_LIST[i] == id) {
+			FIPS_OVERRIDE_LIST[i] = -1;
+			goto done;
+		}
+	}
+
+done:
+	if (pthread_rwlock_unlock(&fips_list_lock) != 0)
+		syslog(LOG_ERR, "Error unlock write-lock in remove_from_fips_override_list");
+}
 #endif /* FIPS_H */
