@@ -66,56 +66,13 @@ unsigned int ica_aes_gcm_initialize_internal(const unsigned char *iv,
 				unsigned int direction);
 
 /*
- * List of non-fips-approved algorithms
+ * Routines that work on the fips algo lists.
  */
-static const int FIPS_BLACKLIST[] = {DES_ECB, DES_CBC, DES_CBC_CS, DES_OFB,
-    DES_CFB, DES_CTR, DES_CTRLST, DES_CBC_MAC, DES_CMAC, P_RNG, DES3_ECB,
-    DES3_CBC, DES3_CBC_CS, DES3_OFB, DES3_CFB, DES3_CTR, DES3_CTRLST,
-    DES3_CBC_MAC, DES3_CMAC, ED25519_KEYGEN, ED25519_SIGN, ED25519_VERIFY,
-    ED448_KEYGEN, ED448_SIGN, ED448_VERIFY, X25519_KEYGEN, X25519_DERIVE,
-    X448_KEYGEN, X448_DERIVE, RSA_ME, RSA_CRT, SHA512_DRNG };
-static const size_t FIPS_BLACKLIST_LEN
-	= sizeof(FIPS_BLACKLIST) / sizeof(FIPS_BLACKLIST[0]);
-
-/*
- * FIPS service indicator: List of tolerated but non-approved algorithms.
- */
-static const int FIPS_OVERRIDE_LIST[] = { RSA_ME, RSA_CRT, SHA512_DRNG };
-static const size_t FIPS_OVERRIDE_LIST_LEN
-	= sizeof(FIPS_OVERRIDE_LIST) / sizeof(FIPS_OVERRIDE_LIST[0]);
-
-/*
- * Returns 1 if the algorithm identified by @id is FIPS approved.
- * Returns 0 otherwise.
- */
-static inline int
-fips_approved(int id)
-{
-        size_t i;
-
-        for (i = 0; i < FIPS_BLACKLIST_LEN; i++) {
-                if (id == FIPS_BLACKLIST[i])
-                        return 0;
-        }
-
-        return 1;
-}
-
-/*
- * Returns 1 if the algorithm identified by @id is FIPS tolerated, i.e. it is
- * available via the API in fips mode, but considered non-approved.
- * Returns 0 otherwise.
- */
-static inline int fips_override(int id)
-{
-	size_t i;
-
-	for (i = 0; i < FIPS_OVERRIDE_LIST_LEN; i++) {
-		if (id == FIPS_OVERRIDE_LIST[i])
-			return 1;
-	}
-
-	return 0;
-}
+int fips_approved(int id);
+int fips_override(int id);
+void add_to_fips_black_list(int id);
+void add_to_fips_override_list(int id);
+void remove_from_fips_black_list(int id);
+void remove_from_fips_override_list(int id);
 #endif /* FIPS_H */
 #endif /* ICA_FIPS */
